pro goes_reducer, t1, t2, goes_time, goes_out, total_energy, 	$
	one=one, three=three, five=five, noback=noback, plot=plot, notemps=notemps,$
	fix_time=fix_time, quiet=quiet, first_back=first_back,		$
	threshold=threshold, minratio=minratio, maxratio=maxratio,	$
	_extra=_extra

on_error, 2
COMPILE_OPT hidden

;+
; NAME:
;	goes_reducer
; PURPOSE:
;	Reduce GOES data to T, em, luminosity, etc. Note that the default
;	call gets 3-second data from the GOES archive. These data, while
;	good, often have "range change" glitches arising in the GOES
;	analog electronics. Right now these must be dealt with manually
;	if one needs a clean time series.
; CATEGORY:
;	X-rays: GOES
; CALLING SEQUENCE:
;	goes_reducer, t1, t2, goes_time, goes_out, total_energy
;	goes_reducer, t1, t2, goes_time, goes_out, total_energy, /three
;	goes_reducer, t1, t2, goes_time, goes_out, total_energy, /noback,/plot
;	goes_reducer, t1, t2, goes_time, goes_out, /notemps
;	goes_reducer, t1, t2, goes_time, goes_outs, /three, /fix_time, /noback, /quiet
;
; INPUTS:
;	t1, t2			define GOES time period
;					these times are passed to read procedure 'rd_gxd'.
;					Usually t1 and t2 will be start and end time as a formatted
;					string of type '15-nov-91 22:00'. Alternatively, setting t1
;					to a week number and t2 to a year reads a whole weekly GOES file.
;
; OPTIONAL INPUT PARAMETERS:
;	/noback			defeats the background estimate (i.e, takes the total signal)
;	/first_back		sets the background level to the first point (t1)
;	/plot			makes standard plots (done indepently with GR_SUMMARY_PLOT)
;	/fix_time		bypasses the PLOT_LCUR calls, i.e. non-interactive
;	/quit			defeats message-printing (the "Blame Roger Thomas" one)
;	/notemps		just returns data (can be used with /noback just to get
;						the GOES fluxes, as an alternative to plot_goes)
;	/one			reverts to one-minute integrations
;	/five			gives five-minute integrations
;	/three			doesn't do anything except print a note about the default 3-sec data.
;
;	The following three keywords control identification of 'bad data'.
;	T and EM are not calculated (set to NaN) if the GOES fluxes drop below the
;	threshold fluxes; or the GOES ratio is below the minimum ratio or above the
;	maximum ratio (in gr_summary_plot these data are not plotted).
;
;	minratio=minratio
;					minimum ratio of high/low energy GOES channel (default: 0.01)
;	maxratio=maxratio
;					maximum ratio of high/low energy GOES channel (default: 0.7)
;	threshold=threshold
;					2-element array with GOES fluxes for low and high channel
;					(default: [0,0])
;
;	Note that all keywords to plot_lcur, goes_tem, tem_thermal_power and gr_summary_plot
;	are also accepted. See those routines for more details.
;
;	goes_tem keywords			: /thomas, /garcia
;	tem_thermal_power keywords	: /mewe
;	gr_summary_plot keywords	: unit, power and noerror + any old plot keyword
;
; OUTPUTS:
;	goes_time		index-type file of times, for UTPLOT entry
;	goes_out		structure containing the fit parameters:
;						temperatures (MK)
;						emission measures (units of 10^49 cm^-3)
;						total radiated powers (ergs/s)
;							(as calculated by function tem_thermal_power)
;						radio ff flux (sfu @ 15 GHz. This assumes He/H = 6.3% abundance)
;						GOES fractional fluxes (high-energy channel, low-energy channel)
;							(these are the GOES energy flux normalized to the total power)
;						GOES input fluxes (w/m^2; these are background-corrected)
;						GOES backgrounds as input interactively
;						terrmin is lower limit for temperature
;						terrmax is upper limit for temperature
;						emerrmin is lower limit for emission measure
;						emerrmax is upper limit for emission measure
;	total_energy	the time integral of X-ray radiated energy
; WARNING:
;	The present GOES data sometimes have mismatched points in the different channels.
;	This causes a 'dischronicity alert' and a halt.
;	Beware of results if the signal-to-background ratio is small. The total power fit
;	is valid only for 5-30 MK and doesn't include long-wavelength emissions of
;	probable significance, such as Ly-alpha or He 304.
;	With the 3-sec GOES data, the "range change" glitches are quite obvious. These
;	must currently be dealt with manually. None of this has been tried on GOES-8 data.
; CALLS:
;	checkvar, rd_gxd, plot_lcur, goes_tem, tem_thermal_power, gr_summary_plot
; MODIFICATION HISTORY:
;	HSH written 21-Feb-93
;	HSH added radio fluxes (algorithm from Stephen White, U. Md.) 13-Mar-93
;	HSH added fractional energy flux outputs 30-Mar-93
;	HSH added /three keyword 18-Apr-93
;	HSH added /noback and fixed other things 24-Apr-93
;	PLB added error bars to the calculation 9/13/93.  see ApJ reference.
;		this uses the three extremes of including and subtracting
;		background fluxes; it does not fit the rise of the flare as
;		described in the reference (Bornmann, P. 1990, Ap. J. 356, p. 733).
;	HSH added /plot keyword 9-Oct-93
;	HSH added /notemps keyword 22-Nov-93
;	HSH added /fix_time, /quiet, /first_back 11-May-95
;	HSH converted to RD_GXD, 3-sec default, 2-Dec-95 (/one, /five)
;	HSH fixed the radio flux algorithm (thanks for bug report from K. Shibasaki, NRO), 26-Apr-96
;	JUNE-2001; Paul Hick (UCSD/CASS; pphick@ucsd.edu)
;		Replaced call to thomas_goes by calls to goes_tem and tem_thermal_power.
;		EM units changed from 10^55 to 10^49 cm^-3 (consistent with goes_tem).
;		Added keywords minratio, maxratio and threshold
;		Added keyword _extra to pass arguments to tem_thermal_power, goes_tem, gr_summary_plot
;-

checkvar, one		, 0
checkvar, three		, 0
checkvar, five		, 0
checkvar, quiet		, 0
checkvar, plot		, 0
checkvar, first_back, 0

checkvar, notemps	, 0		&	get_tem = 1-notemps
checkvar, noback	, 0		&	get_back= 1-noback
checkvar, fix_time	, 0		&	get_time= 1-fix_time

checkvar, minratio	, 0.01
checkvar, maxratio	, 0.70

case n_elements(threshold) of
0: threshold = [.0,.0]
1: threshold = replicate(threshold[0], 2)
endcase



if three then	$
  for i=0,4 do print,'GOES_REDUCER now assumes 3-sec data unless /one or /five are set'


; Read the GOES data

rd_gxd, t1, t2, goes, one=one, five=five, status=status

if status ne 0 then begin
	message, /info, 'rd_gxd returned non-zero error status. Thank you, and good bye'
	return
endif


; Interactive data selection (using plot_lcur)

npnt = n_elements(goes.lo)

if first_back and npnt gt 5 then begin	; simpler treatments of background fluxes
	ss  = indgen(npnt-5)+ 5
	ssb = indgen(5)

endif else if get_time then begin
	print, ' '
	print, 'SELECT DATA INTERVAL'
	print, ' '
	ss = plot_lcur(goes, goes.lo, title='GOES low channel: Select for reduction', _extra=_extra)
	print, ' '
	if get_back then begin
		print, 'SELECT BACKGROUND INTERVAL'
		print, ' '
		ssb = plot_lcur(goes, goes.lo, title='GOES low channel: Select for background', _extra=_extra)
	endif

endif else	$
	ss  = indgen(npnt)


get_back = get_back and n_elements(ssb) gt 0

if get_back then begin
	loback = mean(goes[ssb].lo, /nan)
	hiback = mean(goes[ssb].hi, /nan)
endif else begin
	loback = 0.0
	hiback = 0.0
endelse

if get_back and not quiet then	$
	message, /info, 'Background: low energy ='+strcompress(loback)+' W/m^2;'+	$
		' high energy ='+strcompress(hiback)+' W/m^2'

tt = goes[ss]
lo = goes[ss].lo
hi = goes[ss].hi

goes_out = {goes_params, temp:0.0, em:0.0, power:0.0, radio:0.0, $
	fluxes:[0.,0.],fracs:[0.,0.],backs:[0.,0.], $
	terrmin:0.0, terrmax:0.0, emerrmin:0.0, emerrmax:0.0}


npnt = n_elements(tt)

goes_time = tt
goes_out  = replicate(goes_out,n_elements(tt))

goes_out.backs[0] = hiback
goes_out.backs[1] = loback

goes_out.fluxes = (transpose([[hi],[lo]])-goes_out.backs) > 0

if get_tem then begin

; goes_tem returns temperature in MK and emission measure in 10^49 cm^-3

	all_lo = [lo-loback, lo, lo-loback] > 0
	all_hi = [hi-hiback, hi, hi       ] > 0

	goes_tem, all_lo , all_hi, t6, em49, _extra=_extra

; Decide on which data to plot

	bad = where(all_lo le threshold[0] or all_hi le threshold[1] or	$
				all_hi lt minratio*all_lo or all_hi gt all_lo*maxratio, nbad)

; Flag 'bad data'. These are not plotted by gr_summary_plot

	if bad[0] ne -1 then begin
		t6  [bad] = !values.f_nan
		em49[bad] = !values.f_nan
	endif

	goes_out.temp = t6  [0:npnt-1]
	goes_out.em   = em49[0:npnt-1]


; The min, max values will be finite whe at least one of the three values is finite.

	goes_out.terrmin  = goes_out.temp < t6  [npnt:2*npnt-1] < t6  [2*npnt:3*npnt-1]
	goes_out.terrmax  = goes_out.temp > t6  [npnt:2*npnt-1] > t6  [2*npnt:3*npnt-1]
	goes_out.emerrmin = goes_out.em   < em49[npnt:2*npnt-1] < em49[2*npnt:3*npnt-1]
	goes_out.emerrmax = goes_out.em   > em49[npnt:2*npnt-1] > em49[2*npnt:3*npnt-1]

	goes_out.power	  = tem_thermal_power(goes_out.temp, goes_out.em, _extra=_extra, flux_at_1AU=flux)

	flux = flux*1e-3			; Convert from erg/s/cm^2 to W/m^2

	goes_out.fracs = goes_out.fluxes/ transpose([[flux],[flux]])	; Fractions of total flux
	goes_out.radio = 1.577*goes_out.em/sqrt(goes_out.temp) * (24.5+alog(goes_out.temp*1.e+06)-alog(15.0e+09))
;	goes_out.radio = 1.577*goes_out.em/sqrt(goes_out.temp) * (24.2+alog(goes_out.temp       )-alog(15.0e+09))



; Sum over time to get total energy output from event (erg)

	dt = tt[1:npnt-1].time-tt[0:npnt-2].time
	dt = [dt, dt[npnt-2]]/1000.0					; Time differences in seconds
	total_energy = total(goes_out.power*dt, /nan)	; erg
	message, /info, 'Total radiated energy over'+strcompress(total(dt))+	$
					' seconds is'+strcompress(total_energy)+' erg'

; Make plots of temperature and emission measure results

	if plot then gr_summary_plot, goes_time, goes_out, _extra=_extra

endif

return  &  end