pro gr_summary_plot, times, outs, noerror=noerror, plotorder=plotorder,	$
	unit=unit, power=power, charsize=charsize, _extra=_extra

on_error, 2
COMPILE_OPT hidden

;+
; NAME:
;	gr_summary_plot
; PURPOSE:
;	Make a 4-panel or 6-panel plot showing output from 'goes_reducer'
; CATEGORY:
;	X-rays, GOES
; CALLING SEQUENCE:
;	gr_summary_plot, times, outputs [,unit=unit, /noerror, plotorder=plotorder, /power]
; INPUTS:
;	times
;	outputs
;			output variables from GOES_REDUCER
; OPTIONAL INPUT PARAMETERS:
;	/power	if not set a 2x2 plot is made for the fluxes, T and EM
;			if set a 2x3 plot is made, adding power and radio flux
;	unit=unit
;		4- or 6-element integer array; changes the units on the y-axis
;			unit[0] changes the hard channel  units to 10^unit[0] W/m^2
;			unit[1] changes the soft channel  units to 10^unit[1] W/m^2
;			unit[2] changes the temperature   units to 10^unit[2] K
;			unit[3] changes the emiss measure units to 10^unit[3] cm^-3
;			unit[4] changes the power		  units to 10^unit[4] erg/s
;			unit[5] changes the radio flux    units to 10^unit[5] ???
;		Only the integer part of 'unit' is used
;	/noerrors
;		suppresses plotting of error estimates on the temperature
;		and emission measure plots.
;	plotorder=plotorder
;		4- or 6-element integer array with a permutation of numbers 0,1,2,3,4,5
;		Default is [0,2,1,3,4,5].
;		Modifies the plot layout. By default the top-left plot is
;		the hard channel, bottom-left is the soft channel, top-right is
;		the temperature, and bottom-right is the emission measure.
; CALLS:
;	utplot, outplot, clear_utplot
; RESTRICTIONS:
;	The input units for the GOES fluxes should be to W/m^2, the input
;	temperature should be in MK, the emission measure in 10^55 cm^-3
; OUTPUTS:
;	none
; MODIFICATION HISTORY:
;	HSH, written 9 Oct. 1993
;	JUN-2001, Paul Hick (UCSD/CASS; pphick@ucsd.edu)
;		added keywords 'plotorder', 'unit'
;		!p.multi is saved on entry and restored on exit (rather than setting it to zero).
;		added optional power and radio flux plots
;-

power  =   keyword_set(power  )
errors = 1-keyword_set(noerror)

if n_elements(charsize) eq 0 then charsize = 1.2

nplots = 4+2*power

if n_elements(plotorder) eq 0 then begin
	plotorder = [0,2,1,3]
	if power then plotorder = [plotorder, 4,5]
endif

nplots =  nplots < n_elements(plotorder)

; Input units: (the exponents are stored in defunits)
;	high channel: 10^0 W/m2
;	low  channel: 10^0 W/m2
;	temperature : 10^6 K
;	emiss. meas.: 10^49 cm^-3
;	power		: 10^0 erg/s
;	radio flux	: 10^0

defunit = [0,0,6,49,0,0]
if n_elements(unit) eq 6 then unit = fix(unit) else unit = [-5,-5,6,49,26,0]


title = 'GOES '+['Hard Channel','Soft Channel','Temperature','EM','Power' ,'Radio Flux']
units = ['W/m!E2!N'    ,'W/m!E2!N'    ,'K'          ,'cm!E-3!N'  ,'erg s!E-1!N','']

cunit = ' 10!E'+strcompress(unit,/rem)+'!N'	; Add 10^# to y-title
i = where(unit eq 0)
if i[0] ne -1 then cunit[i] = ''			; Clear 10^0 in y-title

title = title+','+cunit+' '+units

pmulti_old = !p.multi
!p.multi = [0,2,2+power]

for i=0,nplots-1 do begin
	j = plotorder[i]

; If selected units are different from the default units than
; the data arrays need to multiplied by a power of ten.

	factor = 10.0^(defunit[j]-unit[j])

	case j of
	0: f = outs.fluxes[0]
	1: f = outs.fluxes[1]
	2: f = outs.temp
	3: f = outs.em
	4: f = outs.power
	5: f = outs.radio
	endcase

	utplot, times, f*factor, title=title[j], charsize=charsize, _extra=_extra

	if errors and (j eq 2 or j eq 3) then begin
		case j of
		2: begin
			flo = outs.terrmin
			fhi = outs.terrmax
		end
		3: begin
			flo = outs.emerrmin
			fhi = outs.emerrmax
		end
		endcase

		outplot, times, flo*factor, linestyle=1
		outplot, times, fhi*factor, linestyle=1

	endif

endfor

!p.multi = pmulti_old

clear_utplot

return & end